/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.citrusframework.yaks.camelk;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Christoph Deppisch
 */
public final class CamelKHelper {

    /** Logger */
    private static Logger LOG = LoggerFactory.getLogger(CamelKHelper.class);

    private CamelKHelper() {
        // prevent instantiation of utility class.
    }

    /**
     * Retrieve namespace to connect with based on System environment settings.
     * @return
     */
    public static String namespace() {
        String defaultNamespace = CamelKSettings.getNamespace();
        if (defaultNamespace != null) {
            return defaultNamespace;
        }

        final File namespace = new File("/var/run/secrets/kubernetes.io/serviceaccount/namespace");
        if (namespace.exists()){
            try {
                return new String(Files.readAllBytes(namespace.toPath()), StandardCharsets.UTF_8);
            } catch (IOException e) {
                LOG.warn("Failed to read namespace from file {}", namespace, e);
            }
        }

        return null;
    }
}
